package type3;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import auxiliary.ListofListContains;

public class Type3Parser {

	private static FiniteStateMachine grammarToFSM(Map<String, List<List<String>>> rules, String start) {
		Map<String, State> states = new HashMap<>();
		for (Entry<String, List<List<String>>> entry : rules.entrySet()) {
			if (entry.getKey().equals(start) && ListofListContains.contains(entry.getValue(), new LinkedList<>())) {
				states.put(start, new State(start, true));
			} else {
				states.put(entry.getKey(), new State(entry.getKey()));
			}
		}
		String stop = "qStop";
		states.put(stop, new State(stop, true));
		for (Entry<String, List<List<String>>> entry : rules.entrySet()) {
			for (List<String> c : entry.getValue()) {
				if (c.equals(new LinkedList<>())) {
					continue;
				}
				if (c.size() == 2) {
					states.get(entry.getKey()).with(new Transition(c.get(0), states.get(c.get(1))));
				} else {
					states.get(entry.getKey()).with(new Transition(c.get(0), states.get(stop)));
				}

			}
		}
		return new FiniteStateMachine(states.get(start + ""));
	}

	public static boolean isPartOfLanguage(Map<String, List<List<String>>> rules, String word, String start) {
		FiniteStateMachine fsm = grammarToFSM(rules, start);
		for (char c : word.toCharArray()) {
			fsm = fsm.switchStates(c + "");
		}
		return fsm.canStop();
	}

	public static void main(String[] args) {
		Map<String, List<List<String>>> rules = new HashMap<>();
		List<List<String>> s = List.of(List.of(), List.of("a", "A"), List.of("b", "B"), List.of("a", "C"),
				List.of("a"));
		rules.put("S", s);
		List<List<String>> a = List.of(List.of("b", "Q"));
		rules.put("A", a);
		List<List<String>> q = List.of(List.of("b", "B"));
		rules.put("Q", q);
		List<List<String>> b = List.of(List.of("a", "A"), List.of("b"), List.of("a"));
		rules.put("B", b);
		List<List<String>> c = List.of(List.of("b", "B"), List.of("c", "C"));
		rules.put("C", c);
		System.out.println(isPartOfLanguage(rules, "", "S"));
	}

}
